local taskName = "call"
local calling = {}

--[[
        CC_STATE            通话状态
        PLAY_DONE           音频播放结束
        CC_READY            通话功能准备OK
        CC_DONE             无论何种原因的通话结束
        INCOMINGCALL        来电
        CONNECTED           通话建立
        CALL_KEY            按键1
        HANGUP_KEY          按键2
]]

audio.on(0, function(id, event)
    -- 使用play来播放文件时只有播放完成回调
    local succ, stop, file_cnt = audio.getError(0)
    if not succ then
        if stop then
            log.info("用户停止播放")
        else
            log.info("第", file_cnt, "个文件解码失败")
        end
    end
    sysplus.sendMsg(taskName, "CC_STATE", "PLAY_DONE")
end)

sys.subscribe("CC_IND", function(state)
    log.info("cc state", state)
    if state == "READY" then -- 通话准备就绪
        sys.publish("CC_READY")
        sysplus.sendMsg(taskName, "CC_STATE", state)
    elseif state == "INCOMINGCALL" then
        log.info("cc.lastNum", cc.lastNum())
        sysplus.sendMsg(taskName, "CC_STATE", state)
    -- 当状态为 "HANGUP_CALL_DONE"（通话结束）、"MAKE_CALL_FAILED"（拨打失败）或 "DISCONNECTED"（通话断开）时，会将音频模块设置为待机状态，调用 audio.pm(0, audio.STANDBY)。
    elseif state == "HANGUP_CALL_DONE" or state == "MAKE_CALL_FAILED" or state == "DISCONNECTED" then
        sysplus.sendMsg(taskName, "CC_STATE", "CC_DONE")
        audio.pm(0,audio.STANDBY)
        -- audio.pm(0,audio.SHUTDOWN)   --低功耗可以选择SHUTDOWN或者POWEROFF，如果codec无法断电用SHUTDOWN
    elseif state == "CONNECTED" then
        sysplus.sendMsg(taskName, "CC_STATE", "CONNECTED")
    end
end)

local list = {{
    name = "测试电话1",
    num = "xxx"
}, {
    name = "测试电话2",
    num = "xxx" 
}}
-- 查找给定电话号码在联系人列表中的位置
local function findVaildNum(list, num)
    for index, v in ipairs(list) do
        if num == v.num then
            log.info("find vaild num", index, v.name, v.num)
            return index
        end
    end
    return false
end

sysplus.taskInitEx(function()
    local es8311i2cId = 0       -- I2CID
    local es8311PowerPin = 2    -- ES8311电源控制引脚
    local paPin = 23            -- PA放大器控制引脚

    mcu.altfun(mcu.I2C, es8311i2cId, 13, 2, 0)
    mcu.altfun(mcu.I2C, es8311i2cId, 14, 2, 0)

    i2c.setup(es8311i2cId, i2c.FAST)
    gpio.setup(2, 1)
    sys.wait(20)
    if i2c.send(0, 0x18, 0xfd) == true then
        log.info("音频小板或内置ES8311", "codec on i2c0")
        i2c_id = 0
        find_es8311 = true
    end

    if not find_es8311 then
        while true do
            log.info("not find es8311")
            sys.wait(1000)
        end
    end
    i2s.setup(0, 0, 16000, 16, i2s.MONO_R, i2s.MODE_LSB, 16)

    audio.config(0, paPin, 1, 3, 100, es8311PowerPin, 1, 100)
    audio.setBus(0, audio.BUS_I2S, {
        chip = "es8311",
        i2cid = es8311i2cId,
        i2sid = 0,
        voltage = audio.VOLTAGE_1800
    }) -- 通道0的硬件输出通道设置为I2S

    audio.vol(0, 80)        -- 喇叭输出音量
    audio.micVol(0, 80)     -- mic输入音量
    audio.debug(true)
    log.info("cc初始化", cc.init(0))
    audio.pm(0, audio.STANDBY)
    -- sys.waitUntil("CC_READY")

    local mode, index, ret = "IDLE", 0, nil
    local ttsDone = true
    local isSelect = false
    while true do
        log.info("mode", mode)
        if mode == "IDLE" then
            log.info("IDLE")
            index = 1
            ret = sysplus.waitMsg(taskName, nil, nil)
            log.info("IDLE", ret[1], ret[2])
            if ret[2] == "INCOMINGCALL" then -- 来电
                local num = cc.lastNum() -- 获取来电号码
                log.info("来电", num)
                index = findVaildNum(list, num) -- 如果号码有效，则进入准备通话状态，无效则挂断
                if index then
                    mode = "PREPARE"
                    local result = audio.tts(0, list[index].name .. "电话")
                    log.info("tts result", result)
                else
                    mode = "DISCONNECTING"
                end
            elseif ret[2] == "CALL_KEY" then -- 主动拨号
                log.info("CALL_KEY")
                mode = "WAIT_CALLING"
            elseif ret[2] == "PLAY_DONE" then
                ttsDone = true
            end
        elseif mode == "WAIT_CALLING" then -- 等待呼叫
            log.info("WAIT_CALLING")
            ret = sysplus.waitMsg(taskName, nil, 4000)
            if not ret then -- 等待超时，拨打当前选中的联系人，默认为第一个
                if not isSelect then
                    isSelect = true
                    audio.tts(0, "正在拨打" .. string.fromHex(list[index].name) .. "的电话")
                end
                log.info("WAIT_CALLING timeout, dial", list[index].name, list[index].num)
                cc.dial(0, list[index].num)
                mode = "CALLING"
            elseif ret[2] == "CALL_KEY" then -- 按下拨号键，则选择下一个联系人
                if ttsDone then
                    index = index + 1
                    if index > #list then
                        index = 1
                    end
                    ttsDone = false
                    audio.playStop(0)
                    local result = audio.tts(0, "即将呼叫" .. string.fromHex(list[index].name))
                    log.info("即将呼叫", list[index].name, list[index].num)
                end
            elseif ret[2] == "HANGUP_KEY" then -- 按下挂断键，挂断电话
                audio.playStop(0)
                mode = "DISCONNECTING"
            elseif ret[2] == "PLAY_DONE" then
                ttsDone = true
                if isSelect then
                    isSelect = false
                    mode = "CALLING"
                    cc.dial(0, list[index].num)
                end
            elseif ret[2] == "CC_DONE" then -- 通话结束，回到IDLE
                audio.playStop(0)
                mode = "IDLE"
            end
        elseif mode == "CALLING" then -- 拨号中
            ret = sysplus.waitMsg(taskName, nil, nil)
            if ret[2] == "CC_DONE" or ret[2] == "HANGUP_KEY" then -- 通话结束或挂断键按下，走到挂断流程
                mode = "DISCONNECTING"
            elseif ret[2] == "CONNECTED" then -- 收到通话建立消息，则进入通话中状态
                mode = "CONNECTING"
            end
        elseif mode == "PREPARE" then -- 准备通话
            ret = sysplus.waitMsg(taskName, nil, nil)
            log.info("PREPARE", ret[1], ret[2])
            if ret[2] == "CALL_KEY" then -- 按下拨号键，接听，则进入通话中状态
                audio.playStop(0)
                cc.accept(0)
                mode = "CONNECTING"
            elseif ret[2] == "HANGUP_KEY" then -- 按下挂断键，走到挂断流程
                audio.playStop(0)
                mode = "DISCONNECTING"
            elseif ret[2] == "PLAY_DONE" then -- 播放完毕，循环播放
                ttsDone = true
                local result = audio.tts(0, string.fromHex(list[index].name .. "电话"))
                log.info("tts result", result)
            elseif ret[2] == "CC_DONE" then -- 通话结束，回到IDLE
                audio.playStop(0)
                mode = "IDLE"
            end
        elseif mode == "CONNECTING" then -- 通话中
            ret = sysplus.waitMsg(taskName, nil, nil)
            log.info("CONNECTING", ret[1], ret[2])
            if ret[2] == "HANGUP_KEY" then -- 按下挂断键，走到挂断流程
                mode = "DISCONNECTING"
            elseif ret[2] == "CC_DONE" then -- 通话结束，回到IDLE
                mode = "IDLE"
            end
        elseif mode == "DISCONNECTING" then -- 挂断流程
            cc.hangUp(0) -- 挂断电话
            ret = sysplus.waitMsg(taskName, nil, 10000)
            log.info("DISCONNECTING", ret[1], ret[2])
            if not ret or ret[2] == "CC_DONE" then -- 通话结束或者超时没有等到挂断结束的消息，回到IDLE
                mode = "IDLE"
            elseif ret[2] == "PLAY_DONE" then
                ttsDone = true
            end
        end
    end
end, taskName)

-- 按键1 呼叫
gpio.debounce(28, 1000)
gpio.setup(28, function()
    log.info("callme", "拨号键触发, 拨打电话")
    sysplus.sendMsg(taskName, "CC_STATE", "CALL_KEY")
end, gpio.PULLUP)

-- 按键2 挂断
gpio.debounce(18, 1000)
gpio.setup(18, function()
    log.info("callme", "挂断键触发, 挂断电话")
    sysplus.sendMsg(taskName, "CC_STATE", "HANGUP_KEY")
end, gpio.PULLUP)


return calling
